{% extends 'core_main_app/user/sharing/button.html' %}

{% block btn_id %}pid-sharing{% endblock %}
{% block btn_class %}btn btn-secondary float-right mr-1{% endblock %}
{% block btn_icon %}link{% endblock %}
{% block btn_name %}Share PID{% endblock %}
